/*----------------------------------------------------------------
 * CLA.H - CSL basic definitions
 * Copyright (C) 198?-199? TopSpeed Software Corporation
 * All rights reserved
 *----------------------------------------------------------------*/
#ifndef _CLA_INCL
#define _CLA_INCL
#include <stdarg.h>
#include <clacomp.hpp>
#include <clapic.hpp>
#include <claq.hpp>

#ifdef __cplusplus
extern "C" {
#endif
#pragma save,name(prefix=>"")

/*-Macros-------------------------------------------------------------------*/
#define forever for(;;)
#define FALSE      0
#define TRUE       1
#define ERR        -1
#define LERR       0xffffffffL
#define OK  0
#define STRMAX     255

/*--------------------------------------------------------------------------*/


/**/
/* Control Statements                                                       */
/**/

    void        Cla$RUN(/* string, */ int=0, int=0);
    int         Cla$CALL(/* string, string */ UINT);
    void        Cla$CHAIN(/* string */ void);
    long        Cla$RUNCODE(void);
    void        Cla$UNLOAD(/* string*/ void);

    void        Cla$SHUTDOWN(void (*func)(void));
    void        Cla$code(void);
    void        Cla$CloseAll(void);

    long        Cla$STATUSfile( Cla_FCBBLK *fcb );
    void /*string*/ Cla$ERRORFILE(void);

/**/
/* Memory Table Processing And Misc Functions                               */
/**/

#define        GETMEM(a)        calloc(1,a)

/**/
/* Mathematical Functions                                                   */
/**/

    double      Cla$INT(double x);
    double      Cla$LOGE(double x);
    double      Cla$LOG10(double x);
    long        Cla$RANDOM(long low,long high);
    double      Cla$ROUND(double x,double order);
    double      Cla$SQRT(double x);

/**/
/* Trigonometric Functions (in radians)                                     */
/**/

    double      Cla$ACOS(double radians);
    double      Cla$ASIN(double radians);
    double      Cla$ATAN(double radians);
    double      Cla$COS(double radians);
    double      Cla$SIN(double radians);
    double      Cla$TAN(double radians);

/**/
/* Bit Manipulation Functions                                               */
/**/

    long         Cla$BSHIFT(long x,int count);

/**/
/* Miscellaneous compiler helper functions                                  */
/**/

    double      Cla$longpower(double x,long y);
    double      Cla$realpower(double x,double y);
    double      Cla$modulus(double x,double y);
    int         Cla$realdistinct(double x, double y);
    int         Cla$srealdistinct(float x, float y);
    double      Cla$divide(double x, double y);
    long        Cla$lroundd(double x);
    unsigned long Cla$uroundd(double x);
    long        Cla$lroundf(float x);
    unsigned long Cla$uroundf(float x);

/**/
/* String Functions                                                         */
/**/

    StrDesc     Cla$ALL(char *string,unsigned slen,unsigned dlen);
    StrDesc     Cla$SUB(char *string,unsigned slen,int pos,unsigned len);
    unsigned    Cla$clipstr(char *string);                        /* clastr   */

/**/
/* Data Type Conversion/Handling Functions                                  */
/**/

    void        Cla$freestr(StrDescP s);
    StrDesc     Cla$loadstr(char *s,unsigned len);
    void        Cla$storestr(char *d, unsigned dl, char *s, unsigned sl);
    void        Cla$storecstr(char *dst,unsigned dlen,void *src,unsigned slen);
    void        Cla$storepstr(char *dst,unsigned dlen,void *src,unsigned slen);
    int         Cla$comparestr(char *s1,unsigned s1l,char *s2,unsigned s2l);
    StrDesc     Cla$concatstr(char *s1,unsigned s1l,char *s2,unsigned s2l);
    StrDesc     Cla$longtostr(long l);
    double      Cla$strtoreal(const char *string,unsigned len);
    StrDesc     Cla$realtostr(double v);
    double      Cla$loaddec(void *deciptr,unsigned char declen,unsigned char dplace);
    void        Cla$storedec(void *d,unsigned char declen,unsigned char dplace,double v);
    double      Cla$ploaddec(void *deciptr,unsigned char declen,unsigned char dplace);
    void        Cla$pstoredec(void *d,unsigned char declen,unsigned char dplace,double v);
    long        Cla$pictloadlong(const char *s, unsigned l, const char *ptok);
    double      Cla$pictloadreal(const char *s, unsigned l, const char *ptok);
    void        Cla$pictstorelong(char *s, unsigned l, const char *ptok,long v);
    void        Cla$pictstorereal(char *s, unsigned l, const char *ptok,double v);
    void        Cla$duplicate(char *dst,unsigned dlen,unsigned count);

    char       *Cla$deformatpict(const char *s, unsigned l, const char *ptok);

    StrDesc     Cla$DEFORMAT(const char *string,unsigned slen,const char *pic);
    StrDesc     Cla$FORMATstr(const char *string,unsigned slen,const char *pic);

    double      Cla$bloadfloat(void *in);
    void        Cla$bstorefloat(void *in,double d);
    double      Cla$bloadfloat4(void *in);
    void        Cla$bstorefloat4(void *in,double d);

/**/
/* Date Functions                                                           */
/**/

    StrDesc	Cla$AGE(long bday,long base);
    long	Cla$DATE(unsigned mo,unsigned dy,unsigned yr);
    long	Cla$DAY(long cdate);
    long	Cla$MONTH(long cdate);
    long	Cla$TODAY(void);
    long	Cla$YEAR(long cdate);
    void	Cla$SETTODAY(long cdate);

    typedef struct {
	unsigned char	dy;
	unsigned char	mo;
	unsigned short  yr;
    } btdate;

    long	Cla$loadbtdate(long date);
    long	Cla$storebtdate(long date);

    int		Cla$valabs(long absday);
    int		Cla$valdate(unsigned month,unsigned day,unsigned year);
    long	Cla$date2abs(struct date *d);
    int		Cla$abs2date(long absday,struct date *d);
    int         Cla$Intellidate(int yr,char years_before = 80);
    int         Cla$IsLeapYear(unsigned year);

/**/
/* Time Functions                                                           */
/**/

    long	Cla$CLOCK(void);
    void	Cla$SETCLOCK(long ctime);

    typedef struct {
	char	    hun;
	char	    sec;
	char	    min;
	char	    hour;
    } bttime;

    long	Cla$loadbttime(long time);
    long	Cla$storebttime(long time);
    long	Cla$time2abs(struct time *tm);
    int		Cla$abs2time(long abstim,struct time *tm);

/**/
/* Error Handling Functions                                                 */
/**/

extern unsigned     Cla$DOSerror;
extern int          Cla$ERRCODE;              /* global error number field */
extern char         Cla$ERRSTR[80];           /* global error description  */
extern Cla_FCBBLK  *Cla$FileError;           // pointer to fcb of last file w/error

    void        Cla$seterror(int err);
    void        Cla$pusherrcodes();
    void        Cla$poperrcodes();
    void        Cla$clearerror(void);
    void        Cla$rterr(int err,void *msg);
    char       *Cla$err_message(unsigned err);
    void        Cla$AssertFailed(unsigned line, char *filename);
    void        Cla$setfileerror(char *, char *);
    void        Cla$FILEERRORMSG(void);
    void        Cla$FILEERRORCODE(void);
    void        Cla$SetErrorFile(Cla_FCBBLK  *File);

/**/
/* Miscellaneous Statements                                                 */
/**/

#ifndef __32BIT__
    unsigned    Cla$Seg2Sel(unsigned seg);
#endif
    void	Cla$DELAY(unsigned millisecs);
    void	Cla$BEEP(unsigned freq,unsigned dur);
    void	Cla$IN(unsigned port,unsigned *input,unsigned bytes);
    void	Cla$OUT(unsigned port,void *output,unsigned bytes);
    void	Cla$PEEK(unsigned long a,void *r, unsigned len);
    void	Cla$POKE(unsigned long a,void *v, unsigned len);

    unsigned long Cla$crc32(char *buf, unsigned len, unsigned long crc);

    void        Cla$DIRECTORY(QUEUECB * tcb, unsigned char attrib );
    void        Cla$LFNDIRECTORY(QUEUECB * tcb, unsigned char attrib );
    void /*string*/ Cla$LONGPATH(unsigned char /*string*/);
    void /*string*/ Cla$SHORTPATH(unsigned char nopath /*string*/);
    void	Cla$SETPATH(void);
    int         Cla$fnsplit(char *path, char *drv, char *dir, char *name, char *ext );

/**/char *	Cla$ERROR(char *dest,unsigned dlen);
/**/unsigned	Cla$ERRORCODE(void);

/**/
/* International Sort                                                       */
/**/

typedef struct {
    unsigned char   collate[256];
    unsigned char   alphacase[256];
    unsigned char   caseflg[256];
    char            mo[12][4];
    char            month[12][13];
    char            am[3];
    char            pm[3];
    char            use;
    char            digraph[16][4];
    char            oemdigraph[16][4];
// beneath are for windows only
    unsigned char   oemcollate[256];
    char            ShortDatePdesc[INTL_MAX_SHORT_DATE_WIDTH*2+4];
    char            LongDatePdesc[INTL_MAX_LONG_DATE_WIDTH*2+4];
    char            ShortTimePdesc[INTL_MAX_SHORT_TIME_WIDTH*2+4];
    char            LongTimePdesc[INTL_MAX_LONG_TIME_WIDTH*2+4];
    char            WinAmPm[2][3];

} ISFinfo;

extern ISFinfo Isfinfo;

ISFinfo *Cla$isfinfo(void);
void Cla$isfinit(void);
int  Cla$isfisalpha(unsigned char c);
int  Cla$isfisalnum(unsigned char c);
int  Cla$isfislower(unsigned char c);
int  Cla$isfisupper(unsigned char c);
char Cla$isftoupper(unsigned char ch);
char Cla$isftolower(unsigned char ch);
int  Cla$isfcompare(const void *src1,const void *src2,unsigned len);
int  Cla$isfoemcompare(const void *src1,const void *src2,unsigned len);
int  Cla$isfcomparec(unsigned char,unsigned char);
int  Cla$isficompare(const void *src1,const void *src2,unsigned len);
char *Cla$isflower(char *ch,unsigned len);
char *Cla$isfupper(char *ch,unsigned len);
char *Cla$isfconvert(char *ch,unsigned len);
char *Cla$isfoemconvert(char *ch,unsigned len);
int  Cla$isfnextstr(char *str, unsigned len);

/**/
/* Option Arguments                                                         */
/**/

void /* string */ Cla$COMMAND(unsigned char /* <string> */ );
void     Cla$SETCOMMAND( /* string */ void);

/**/
/* Transaction Logging handlers                                             */
/**/

void Cla$LOGOUT(int seconds, ... );
void Cla$ROLLBACK(void);
void Cla$COMMIT(void);

void Cla$activate_drv(  void *fcb,
                        int acc_mode,
                        int logoutop,
                        int rollbackop,
                        int commitop,
                        int lockop,
                        int unlockop);

void Cla$deactivate_drv(void *fcb);

#pragma restore
#ifdef __cplusplus
}
#endif

#define _PROT  // for windows
#endif // _CLA_INCL

