/* Release 4.00 */
/*-------------------------------------------------------------------------*
*                                                                          *
*  CLWCLIB.H - Windows 16 bit C library                                   *
*                                                                          *
*  COPYRIGHT (C) 1989..1992 Clarion Software Corporation.                  *
*  All Rights Reserved                                                     *
*                                                                          *
*--------------------------------------------------------------------------*/
#ifndef _CLIB16_INC_
#define _CLIB16_INC_

#ifdef __cplusplus
extern "C" {
#endif


typedef unsigned size_t;
typedef int      ssize_t;

#ifndef _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#define _WCHAR_T_DEFINED
#endif

#ifndef NULL
#ifdef __32BIT__
#define NULL   0
#else
#define NULL   0
#endif
#endif

/*------------------------------------------------------------------------*/
/*----------- Conversions ------------------------------------------------*/
/*------------------------------------------------------------------------*/
double        atof(const char *_nptr);
int           atoi(const char *_nptr);
long          atol(const char *_nptr);
unsigned long atoul(const char *_nptr);
double        strtod(const char *_s, char **_endptr);
long          strtol(const char *_s, char **_endptr, int _base);
long double strtold(const char * nptr, char * * endptr);
unsigned long strtoul(const char *_s, char **_endptr, int _base);
long double   atofl(const char *_nptr);
char *        ecvt(double _value, int _digits, int *_dec, int *_sign);
char *        fcvt(double _value, int _digits, int *_dec, int *_sign);
char *        gcvt(double _value, int _ndec, char *_buf);
char *        gcvtl(long double _value, int _ndec, char *_buf);
char *        itoa(int _value, char *_string, int _radix);
char *        ltoa(long _value, char *_s, int _radix);
char *        ultoa(unsigned long _value, char *_s, int _radix);

/*------------------------------------------------------------------------*/
/*----------- Misc integer maths -----------------------------------------*/
/*------------------------------------------------------------------------*/

typedef struct {
   long    quot;
   long    rem;
} ldiv_t;

typedef struct {
   int    quot;
   int    rem;
} div_t;

int           abs(int _num);
long          labs(long _j);
div_t         div(int _numer, int _denom);
ldiv_t        ldiv(long _numer, long _denom);


/*------------------------------------------------------------------------*/
/*----------- Limit constants---------------------------------------------*/
/*------------------------------------------------------------------------*/

#define CHAR_BIT                  8
#define CHAR_MIN             (-128)
#define CHAR_MAX                127
#define MB_LEN_MAX                1
#define SCHAR_MAX               127
#define SCHAR_MIN            (-128)
#define UCHAR_MAX               255
#define SHRT_MAX              32767
#define SHRT_MIN         (-32767-1)
#define USHRT_MAX            65535U
#define LONG_MAX        2147483647L
#define LONG_MIN   (-2147483647L-1)
#define ULONG_MAX      4294967295UL

#ifdef __32BIT__
#define UINT_MAX          ULONG_MAX
#define INT_MAX            LONG_MAX
#define INT_MIN            LONG_MIN
#else
#define UINT_MAX          USHRT_MAX
#define INT_MAX            SHRT_MAX
#define INT_MIN            SHRT_MIN
#endif

/*------------------------------------------------------------------------*/
/*----------- Char type functions ----------------------------------------*/
/*------------------------------------------------------------------------*/


int toupper(int c);
int tolower(int c);
int isalnum(int c);
int isalpha(int c);
int islower(int c);
int isupper(int c);
int isascii(int c);
int iscntrl(int c);
int isdigit(int c);
int isgraph(int c);
int isprint(int c);
int ispunct(int c);
int isspace(int c);
int isxdigit(int c);

#ifdef _USE_CTYPE_ARRAY
#define _IS_SP      1            /* is space */
#define _IS_DIG     2            /* is digit indicator */

#ifndef _NO_INTL_SUPPORT
#define _IS_UPP     4            /* is upper case */
#define _IS_LOW     8            /* is lower case */
#endif

#define _IS_HEX    16            /* [A-F or [a-f] */
#define _IS_CTL    32            /* Control */
#define _IS_PUN    64            /* punctuation */

extern    char  _ctype[384];            /* Character type array */

#define isascii(c)      ((unsigned)((c)) < 0x80)
#define iscntrl(c)      (_ctype[(c)+128] & _IS_CTL)
#define isdigit(c)      (_ctype[(c)+128] & _IS_DIG)
#define ispunct(c)      (_ctype[(c)+128] & _IS_PUN)
#define isspace(c)      (_ctype[(c)+128] & _IS_SP)
#define isxdigit(c)     (_ctype[(c)+128] & (_IS_DIG | _IS_HEX))
#endif

#ifndef _NO_INTL_SUPPORT
#pragma save,name(prefix=>"")

int  Cla$isfisalpha(unsigned char c);
int  Cla$isfisalnum(unsigned char c);
int  Cla$isfislower(unsigned char c);
int  Cla$isfisupper(unsigned char c);
char Cla$isftoupper(unsigned char ch);
char Cla$isftolower(unsigned char ch);
char *Cla$isflower(char *ch,unsigned len);
char *Cla$isfupper(char *ch,unsigned len);

#pragma restore

#define toupper(c)      Cla$isftoupper(c)
#define tolower(c)      Cla$isftolower(c)
#define isalnum(c)      Cla$isfisalnum(c)
#define isalpha(c)      Cla$isfisalpha(c)
#define islower(c)      Cla$isfislower(c)
#define isupper(c)      Cla$isfisupper(c)
#endif

/*------------------------------------------------------------------------*/
/*----------- Misc functions ---------------------------------------------*/
/*------------------------------------------------------------------------*/
int           atexit(void (*_func)(void));
void	      exit(int _code);
void	      _exit(int _code);
void          delay(int _tm);

#define offsetof(st, member) (size_t)(&((st*)0)->member)
#define _elementsin(a) (sizeof(a)/sizeof((a)[0]))
#ifndef __32BIT__
#define FP_SEG(fp) ((unsigned)(((unsigned long)(fp))>>16))
#define FP_OFF(fp) ((unsigned)(fp))
#pragma save,call(reg_param=>(ax,bx),reg_saved=>(cx,dx,di,si,ds,st1,st2), reg_return=>(bx,es), inline=>on)
inline void *__i_mk_fp(unsigned short,unsigned short) = {0x8e,0xc0,0xcb};
#define MK_FP(seg,off) __i_mk_fp((unsigned short) (seg),(unsigned short) (off))
#pragma restore
#endif

/*------------------------------------------------------------------------*/
/*----------- Memory Allocation-------------------------------------------*/
/*------------------------------------------------------------------------*/
void *        calloc(size_t _num, size_t _size);
void          free(void *_block);
void *        malloc(size_t _size);
void *        realloc(void *_block, size_t _size);

/*------------------------------------------------------------------------*/
/*----------- Utility Functions-------------------------------------------*/
/*------------------------------------------------------------------------*/
void          qsort(void *_base, size_t _nmemb, size_t _size,
                 int (*_compare)(const void *_e1, const void *_e2));
void *        bsearch(const void *_key, const void *_base,
              size_t _nmemb, size_t _size,
              int ( *_compare)(const void *_e1, const void *_e2));
void *        _badd(const void *_newitem, const void *_base,
              size_t _nmemb, size_t _size,
              int ( *_compare)(const void *_e1, const void *_e2),
              int *);
int           rand(void);
void          randomize(void);
void          srand(unsigned _seed);
#define random(num)    (rand() % (num))

#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#define RAND_MAX 32767


/*------------------------------------------------------------------------*/
/*----------- Block and string functions----------------------------------*/
/*------------------------------------------------------------------------*/

int           memcmp(const void *_s1, const void *_s2, size_t _n);
void *        memcpy(void *_dest, const void *_source, size_t _n);
void *        memset(void *_s, int _c, size_t _n);
char *        strcat(char *_dest, const char *_source);
int           strcmp(const char *_s1, const char *_s2);
int           chrcmp(char _c1,char _c2);
int           strequ(const char *_s1, const char *_s2);
char *        strcpy(char *_dest, const char *_source);
size_t        strlen(const char *_s);
void *        memchr(const void *_s, int _c, size_t _n);
void *        memmove(void *_dest, const void *_source, size_t _n);
void *        memwmove(void *_dest, const void *_source, size_t _n);
char *        strchr(const char *_s, int _c);
size_t        strcspn(const char *_s1, const char *_s2);
char *        strerror(int _errnum);
size_t        strspn(const char *_s1, const char *_s2);
char *        strstr(const char *_s1, const char *_s2);
char *        strtok(char *_s1, const char *_s2);
char *        strpbrk(const char *_s1, const char *_s2);
char *        strrchr(const char *_s, int _c);
int           stricmp(const char *_s1, const char *_s2);
char *        strlwr(char *_s);
char *        strupr(char *_s);
char *        strdup(const char *_s);
char *        strrev(char *_s);
int           memicmp(const void *_s1, const void *_s2, size_t _n);
char *        strncat(char *_dest, const char *_source, size_t _n);
int           strncmp(const char *_s1, const char *_s2, size_t _n);
char *        strncpy(char *_dest, const char *_source, size_t _n);
int           strnicmp(const char *_s1, const char *_s2, size_t _n);

#define       _fmemmove memmove
#define       _fmemcpy  memcpy
#define       _fmemcmp  memcmp
#define       _memclr(s, n)      memset(s, 0, n)
#define       _clear(a)          memset(&a, 0, sizeof(a))
/*------------------------------------------------------------------------*/
/*----------- Var arg functions ------------------------------------------*/
/*------------------------------------------------------------------------*/

#define _va_size(t) ((sizeof(t)+sizeof(int)-1)&~(sizeof(int)-1))

typedef char  *va_list;
#define va_start(ap, plist)    ap = (va_list)&plist + sizeof(plist)
#define va_arg(ap, t)          (*(t*)((ap+=_va_size(t))-_va_size(t)))
#define va_end(ap)             ap = NULL

/*------------------------------------------------------------------------*/
/*----------- SetJmp/LongJmp ---------------------------------------------*/
/*------------------------------------------------------------------------*/
typedef unsigned char TenByteReal[10];

typedef struct {
    unsigned  j_sp;       /* saved X086 registers */
    unsigned  j_ss;
    unsigned  j_flag;
    unsigned  j_cs;
    unsigned  j_ip;
    unsigned  j_bp;
    unsigned  j_di;
    unsigned  j_es;
    unsigned  j_si;
    unsigned  j_ds;
    TenByteReal st1, st2;
} jmp_buf[1];

#pragma save
#pragma call(reg_saved=>(ax,bx,cx,dx,ds,es,si,di,st0,st1,st2,st3,st4,st5,st6))
void  longjmp(jmp_buf _env, int _val);
#pragma call(reg_saved=>(ds,es,si,di,st1,st2), set_jmp=>on)
int   _setjmp(jmp_buf _env);
#pragma restore
#define setjmp(_arg) _setjmp(_arg)

/*------------------------------------------------------------------------*/
/*----------- Stream (file) I/O-------------------------------------------*/
/*------------------------------------------------------------------------*/


#ifndef _IOBUF
typedef struct _iobuf near * STREAM;       /* hidden type */
//typedef struct _iobuf  FILE;
#endif

/* Buffer type to be used as 3rd argument for "setvbuf" function */
#define _IOFBF    0
#define _IOLBF    1
#define _IONBF    2

/* End-of-file constant definition  */
#define EOF    (-1)

/* Constants to be used as 3rd argument for "fseek" function */
#define SEEK_SET    0
#define SEEK_CUR    1
#define SEEK_END    2

void          clearerr(STREAM _st);
int           fclose(STREAM _st);
int           fflush(STREAM _st);
int           fgetc(STREAM _st);
char *        fgets(char *s, int _n, STREAM _st);
int           flushall(void);
STREAM        fopen(const char *_path, const char *_mode);
int           fprintf(STREAM _st, const char *_format, ...);
int           fputc(int _c, STREAM _st);
int           fputs(const char *_s, STREAM _st);
size_t        fread(void *_ptr, size_t _size, size_t _n, STREAM _st);
int           fscanf(STREAM _st, const char *_format, ...);
int           fseek(STREAM _st, long _offset, int _whence);
long          ftell(STREAM _st);
size_t        fwrite(const void *_ptr, size_t _size, size_t _n, STREAM _st);
long          fsize(STREAM _st);
int           ftrunc(STREAM _st,long pos);

void          rewind(STREAM _st);
int           scanf(const char *_format, ...);
int           setvbuf(STREAM _st, char *_buf, int _type, size_t _size);
int           sprintf(char *_s, const char *_format, ...);
int           sscanf(const char *_s, const char *_format, ...);
int           ungetc(int _c, STREAM _st);
int           vfprintf(STREAM _st, const char *_format, va_list _argptr);
int           vfscanf(STREAM _st, const char *_format, va_list _argptr);
int           vsprintf(char *_s, const char *_format, va_list _argptr);
int           vsscanf(const char *_s, const char *_format, va_list _argptr);
int           fileno(STREAM _st);
int           feof(STREAM _st);
int           ferror(STREAM _st);
#define getc(f)        fgetc(f)
#define putc(c,f)      fputc(c,f)

int           rmtmp(void);
STREAM        tmpfile(void);
char *        tmpnam(char *_s);
char *        tempnam(char *_dir, char *_prefix);

/*------------------------------------------------------------------------*/
/*----------- File manipulation ------------------------------------------*/
/*------------------------------------------------------------------------*/
/* Macros for access() */
#define F_OK           0
#define W_OK           2
#define R_OK           4

int           access(const char * path, int access);
int           chmod(const char * path, int mode);
int           remove(const char *_path);
int           rename(const char *_oldname, const char *_newname);

/*------------------------------------------------------------------------*/
/*----------- DOS Error codes --------------------------------------------*/
/*------------------------------------------------------------------------*/

#ifdef __32BIT__
extern int   *_errno__();              /* global error variable */
#define errno *_errno__()
#else
extern int errno;
#endif

#define EZERO       0        /* No Error                           */
#define EINVFN      1        /* Invalid function code              */
#define ENOENT      2        /* File not found                     */
#define ENOPATH     3        /* Path not found                     */
#define EMFILE      4        /* Too many open files                */
#define EACCES      5        /* Access denied                      */
#define EBADF       6        /* Invalid handle                     */
#define E2BIG       7        /* Memory blocks destroyed            */
#define ENOMEM      8        /* Insufficient Memory                */
#define EINVMEM     9        /* Invalid memory block address       */
#define EINVENV    10        /* Invalid environment                */
#define EINVFMT    11        /* Invalid format                     */
#define EINVACC    12        /* Invalid access code                */
#define EINVDAT    13        /* Invalid data                       */
#define ENODEV     15        /* Invalid drive                      */
#define ECURDIR    16        /* Attempt to remove CurDir           */
#define ENOTSAM    17        /* Not same device                    */
#define ENMFILE    18        /* No more files                      */

/* RTL error codes */

#define EINVAL      19        /* Invalid argument                  */
#define ENOTDIR     20        /* Not directory                     */
#define EISDIR      21        /* Is directory                      */
#define ENOEXEC     22        /* Corrupted exec file               */
#define EMLINK      32        /* Cross-device link                 */

#define EDOM        33        /* Math argument                     */
#define ERANGE      34        /* Result too large                  */
#define EDEADLOCK   36        /* file locking deadlock             */
#define EEXIST      80        /* File exists MSDOS 3.0 +           */

#define EDISKFULL   240       /* Disk full on write                */

int dosexterr();

/*------------------------------------------------------------------------*/
/*----------- File path manipulation--------------------------------------*/
/*------------------------------------------------------------------------*/
#define MAXPATH  260      /* maximum length of full pathname       */
#define MAXDRIVE   4      /* maximum length of drive component     */
#define MAXDIR   256      /* maximum length of path component      */
#define MAXFILE  256      /* maximum length of file name component */
#define MAXEXT   256      /* maximum length of extension component */
#define _MAX_PATH  MAXPATH
#define _MAX_DRIVE MAXDRIVE
#define _MAX_DIR   MAXDIR
#define _MAX_FNAME MAXFILE
#define _MAX_EXT   MAXEXT


void    fnmerge(char *_path, const char *_drive,const char *_dir,
                const char *_name, const char *_ext);

#define WILDCARDS  0x01
#define EXTENSION  0x02
#define FILENAME   0x04
#define DIRECTORY  0x08
#define DRIVE      0x10

int     fnsplit(const char *_path, char *_drive, char *_dir,
            char *_name, char *_ext);

typedef char FPATH[MAXPATH];
typedef FPATH *PFPATH;

/*------------------------------------------------------------------------*/
/*----------- Directory functions ----------------------------------------*/
/*------------------------------------------------------------------------*/


int           mkdir(const char *_path);
int           rmdir(const char *_path);
int           getcurdir(int _drive, char *_buf);
char	*     getcwd(char *_buf, int _maxlen);

int           chdir(const char *_path);  // change directory only
int           getdisk(void);
int           setdisk(int _drive);

int           setcwd(const char *path);  // change directory AND drive

#define WILDCARDS  0x01
#define EXTENSION  0x02
#define FILENAME   0x04
#define DIRECTORY  0x08
#define DRIVE      0x10

struct ffblk {
    char ff_reserved[21];
    char ff_attrib;
    int  ff_ftime;
    int  ff_fdate;
    long ff_fsize;
    char ff_name[13];
};

int           findfirst(const char *_path, struct ffblk *_ffblk,int _attrib);
int           findnext(struct ffblk *_ffblk);


/*------------------------------------------------------------------------*/
/*----------- Time functions----------------------------------------------*/
/*------------------------------------------------------------------------*/

struct date
{
  int da_year;
  unsigned char da_day;
  unsigned char da_mon;
};

struct time
{
  unsigned char ti_hour;
  unsigned char ti_min;
  unsigned char ti_sec;
  unsigned char ti_hund;
};

void         gettime(struct time *_tptr);
void         getdate(struct date *_dptr);
void         settime(const struct time *tptr);
void         setdate(const struct date *dptr);

typedef long  time_t;
typedef long  clock_t;
#define CLK_TCK 100
#define CLOCKS_PER_SEC 100

clock_t      clock(void);
time_t       time(time_t *_timer);

#pragma save, name(prefix=>"")
void WslDebug$AssertFailed(const char *assertion, const char *file, long int line);
#pragma restore

#if _RELEASE
#define assert(ignore)  ((void) 0)
#else
#define assert(p)	((p) ? ((void) 0) : (( (void) WslDebug$AssertFailed(\
                          #p, __FILE__, __LINE__))))
#endif

typedef void(*PFV)(void);
void _terminate(void);
void _unexpected(void);
PFV _set_terminate(PFV NewTerminate);
PFV _set_unexpected(PFV NewUnexpected);


#ifdef __cplusplus
}
#endif
#endif
