/* Release 0.01a*/
/*
 ***************************************************************************
 *                                                                         *
 *      iodef.h    - Definitions for <stdio.h> functions                   *
 *                                                                         *
 *      Copyright(C) 1992-92 Clarion Software Corporation.                 *
 *                                                                         *
 *      LastMod 25 Nov 92 GH                                               *
 *      Created 27 Oct 92 GH                                               *
 *                                                                         *
 ***************************************************************************/

/*
 * Define some macros for locking around non-reentrant code.
 * The macros are used instead of direct calls in order to improve
 * the flow of the code.
 * (Un)ProtectStream    - Protect access to a stream
 * (Un)ProtectOSAccess  - Stop multiple-entrance to operating system
 * (Un)ProtectSection   - Protect a non-reentrant routine.
 */
#ifdef _mthread
#ifdef _OS2
#define  ProtectStream(stream)          StreamLock(stream)
#define  UnProtectStream(stream)        StreamUnLock(stream)
#define  ProtectOSAccess()
#define  UnProtectOSAccess()
#define  ProtectSection()               Lock()
#define  UnProtectSection()             UnLock()
#else
#define  ProtectStream(stream)          Lock()
#define  UnProtectStream(stream)        UnLock()
#define  ProtectOSAccess()              Lock()
#define  UnProtectOSAccess()            UnLock()
#define  ProtectSection()               Lock()
#define  UnProtectSection()             UnLock()
#endif
#else
#define  ProtectStream(stream)
#define  UnProtectStream(stream)
#define  ProtectOSAccess()
#define  UnProtectOSAccess()
#define  ProtectSection()
#define  UnProtectSection()
#endif

/* If TRUE Standard files are used */
#define StdFiles 0

typedef int _BufferIdx;         /* Must be signed */

#define _MAX_PBACK       4

#define _IOBUF _iobuf

typedef struct _iobuf {
    unsigned char *  _ptr;        /* current char pointer */
    _BufferIdx       _cnt;        /* buffer free space */
    _BufferIdx       _size;       /* buffer size */
    unsigned char *  _base;	  /* buffer address */
    unsigned short   _flag;	  /* file attrib flag */
    int              _file; 	  /* file handle */

    /* Array for storing extra pull back characters */
    unsigned char    _pback[_MAX_PBACK];
    _BufferIdx       _savedcnt;   /* Saved #chars when pushing back */

    /* Single character buffer for use when there is no allocated buffer.
    ** This field MUST NOT be next to _pback - because of the comparisons
    ** done on _pback.    */
    unsigned char    _SmallBuff;

    int              _tmpnum;     /* 0 if normal, or the number for a */
                                  /* temporary file. */

    long _sem;
    char _ctrl;
    char _scnt;

} FILE;

extern unsigned _FilePool;
typedef struct _iobuf <_FilePool> * STREAM;

/* flags bit definitions */

#define _F_RDWR 0x0003			/* Read/write  */
#define _F_READ 0x0001			/* Read only file */
#define _F_WRIT 0x0002			/* Write only file */
#define _F_UBUF 0x0004			/* User allocated buffer */
#define _F_APP  0x0008			/* Append data */
#define _F_ERR  0x0010			/* Error */
#define _F_EOF  0x0020			/* EOF */
#define _F_BIN  0x0040			/* Binary file */
#define _F_IN   0x0080			/* Data is incoming */
#define _F_OUT  0x0100			/* Data is outgoing */
#define _F_DEV  0x0200			/* File is terminal */
#define _F_NEW  0x0400			/* Newly opened stream */
#define _F_LBUF 0x0800			/* line buffered stream */
#define _F_MKRD 0x1000			/* Before closing, change to read
                                           only */
#define _F_RST  0x2000			/* Reset stream (after seek/flush) */
#define _F_STREAM 0x4000                /* Is this a stream */


/* Default buffer size use by "setbuf" function */
#define BUFSIZ    512            /* Buffer size for stdio */

/* IO specific defines */

typedef struct __FileInfoTag <_FilePool> * __FileInfoPtr;

typedef struct __FileInfoTag {
    int            NumFiles;
    FILE           FileRec[255];
    } __FileInfo;

typedef struct { long offset; } fpos_t;


#define P_tmpdir        "\\"    // tmpnam goes into Windows Temp Dir
#define TMP_MAX      10000

/* Flag values for open  */

#define O_RDONLY	     0
#define O_WRONLY	     1
#define O_RDWR		     2

#define O_CREAT		0x0100	/* create and open file */
#define O_TRUNC		0x0200	/* open with truncation */
#define O_EXCL		0x0400	/* exclusive open */

#define _O_EOF		0x0200 /* set when text file hits ^Z   */

/* A file in append mode may be written to only at its end. */

#define O_APPEND	0x0800	/* to end of file */

/* File mode special bits. User may only read these bits */

#define O_CHANGED	0x1000	/* File has been written */
#define O_DEVICE	0x2000	/* user read only flag	 */
#define O_TEXT		0x4000	/* CR-LF translation	*/
#define O_BINARY	0x8000	/* no translation	*/

/* DOS 3.x file sharing options */

#define SH_NOINHERIT	 0x80
#define SH_COMPAT	 0x00
#define SH_DENYRW	 0x10
#define SH_DENYWR	 0x20
#define SH_DENYRD	 0x30
#define SH_DENYNO	 0x40

#define SH_DENYNONE	SH_DENYNO     /* MS documentation uses both */

#define S_IFMT  0170000  /* file type mask */
#define S_IFDIR  0x4000  /* directory */
#define S_IFCHR  0x2000  /* character device */
#define S_IFREG  0x8000  /* regular file */
#define S_IREAD  0x0100  /* owner may read */
#define S_IWRITE 0x0080  /* owner may write */
#define S_IEXEC  0x0040  /* owner may execute <directory search> */

extern STREAM _HandleToStream(int);



#if 00000
#undef  ProtectStream
#undef  UnProtectStream

static void ProtectStream(STREAM * stream)
{
++stream->_scnt;
}

static void UnProtectStream(STREAM * stream)
{
if (stream->_scnt <= 0)
   fprintf(stdout, "Stream %d has < 0 count\n", stream->_file);
else
   --stream->_scnt;
}
#endif

#include <io.h>

