/* Release 3.10 */
/*-------------------------------------------------------------------------*
*                                                                          *
*  math.h -  floating point definitions and prototypes.                    *
*                                                                          *
*  COPYRIGHT (C) 1989..1992 Clarion Software Corporation.                  *
*  All Rights Reserved.                                                    *
*                                                                          *
*--------------------------------------------------------------------------*/

#ifndef _MATH_INC_
#define _MATH_INC_

#ifndef _ERRNOM_INC_
#define _ERRNOM_INC_

#ifdef __cplusplus
extern "C" {
#endif
#ifdef _mthread
#define errno *_errno__()
#define _doserrno *_errno__()

int *        _errno__(void);
#else
extern int   _doserrno;          /* global DOS error variable */
extern int   errno;              /* global error variable */
#endif
#ifdef __cplusplus
}
#endif
#endif

#define EDOM	33		/* Math argument */
#define ERANGE	34		/* Result too large */


#define DOMAIN       1     /* parameter not in valid domain */
#define SING         2     /* function is incalculable at this place */
#define OVERFLOW     3     /* result exceeds representable range */
#define UNDERFLOW    4     /* result too close to zero to be represented */
#define TLOSS        5     /* total loss of precision (eg. Sine (2**65)) */
#define PLOSS        6     /* partial precision loss */

extern double *        __HUGE_VAL_PTR;
extern long double *   __LHUGE_VAL_PTR;

#define HUGE_VAL    (*__HUGE_VAL_PTR)
#define LHUGE_VAL   (*__LHUGE_VAL_PTR)

#ifdef __cplusplus
extern "C" {
#endif
/*
extern double HUGE;
extern long double LHUGE;
*/
int 	 abs(int _x);
double	 acos(double _x);
double	 asin(double _x);
double	 atan(double _x);
double	 atan2(double _y, double _x);
double	 atof(const char *_s);
double	 ceil(double _x);
double	 cos(double _x);
double	 cosh(double _x);
double	 exp(double _x);
double	 fabs(double _x);
double	 floor(double _x);
double	 fmod(double _x, double _y);
double	 frexp(double _x, int *_exp);
long	 labs(long _x);
double	 ldexp(double _x, int _exp);
double	 log(double _x);
double	 log10(double _x);
double	 modf(double _x, double *_ipart);
double   poly(double _x, int _degree, double _coeffs[]);
double	 pow(double _x, double _y);
double	 sin(double _x);
double	 sinh(double _x);
double	 sqrt(double _x);
double	 strtod(const char *_s, char **_eptr);
double	 tan(double _x);
double	 tanh(double _x);

long double	 acosl(long double _x);
long double	 asinl(long double _x);
long double	 atanl(long double _x);
long double	 atan2l(long double _y, long double _x);
long double	 atofl(const char *_s);
long double	 ceill(long double _x);
long double	 cosl(long double _x);
long double	 coshl(long double _x);
long double	 expl(long double _x);
long double	 fabsl(long double _x);
long double	 floorl(long double _x);
long double	 fmodl(long double _x, long double _y);
long double	 frexpl(long double _x, int *_exp);
long double	 ldexpl(long double _x, int _exp);
long double	 logl(long double _x);
long double	 log10l(long double _x);
long double	 modfl(long double _x, long double *_ipart);
long double      polyl(long double _x, int _degree, long double _coeffs[]);
long double	 powl(long double _x, long double _y);
long double	 sinl(long double _x);
long double	 sinhl(long double _x);
long double	 sqrtl(long double _x);
long double	 strtodl(const char *_s, char **_eptr);
long double	 tanl(long double _x);
long double	 tanhl(long double _x);
#ifdef __cplusplus
}
#endif

struct exception {
   int  type;
   char *name;
   double arg1,
          arg2,
          retval;
};

struct exceptionl {
   int  type;
   char *name;
   long double arg1,
               arg2,
               retval;
};

#ifdef __cplusplus
extern "C" {
#endif
extern  int (* matherr)(struct exception *_x);
extern  int (* matherrl)(struct exceptionl *_x);
double         hypot(double _x, double _y);
long double    hypotl(long double _x, long double _y);
double         pow10(int _p);
void           _fpreset(void);
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
struct _complex {	    /* as used by cabs function */
    double  x,
            y;
};

struct _complexl {	    /* as used by cabsl function */
    long double  x,
                 y;
};
#else
struct complex {	    /* as used by cabs function */
    double  x,
            y;
};

struct complexl {	    /* as used by cabsl function */
    long double  x,
                 y;
};

#endif

#define cabs(z)     (hypot ((z).x, (z).y))
#define cabsl(z)    (hypotl((z).x, (z).y))

/* Constants rounded for 23 decimals. */
#define MA_E		 2.7182818284590452353603
#define MA_LOG2E	 1.4426950408889634073599
#define MA_LOG10E	 0.4342944819032518276511
#define MA_LN2		 0.6931471805599453094172
#define MA_LN10 	 2.3025850929940456840180
#define MA_PI		 3.1415926535897932384626
#define MA_PI_2 	 1.5707963267948966192313
#define MA_PI_4 	 0.7853981633974483096157
#define MA_1_PI 	 0.3183098861837906715378
#define MA_2_PI 	 0.6366197723675813430752
#define MA_1_SQRTPI	 0.564189583547756287
#define MA_2_SQRTPI	 1.12837916709551257
#define MA_SQRT2	 1.4142135623730950488017
#define MA_SQRT_2	 0.7071067811865475244084
#endif
