#include "sqlver.h"
#define ORAVER CLAVER "." SQLVER ".022\0"

/*****************************************************************************
 *
 * History
 *
 * 09 Dec 97  .000              . Set to release version
 * 15 Jul 97  .022              . First beta of Buffered driver
 * 30 Jun 97  .021              . Added Asynchronous reads
 * 21 Jun 97                    . Added bulk reads
 * 18 Jun 97  .020              . Fixed bug where import was bringing in
 *                                the wrong size for PDECIMAL fields
 *                                (twice what it should have been).  This
 *                                had a side effect of screwing up later
 *                                timestamp fields.
 * 04 Jun 97  .019              . Fixed bug in dynamic loading in 16 bit.
 * 03 Jun 97  .018              . Changed logon dialog to use registry
 *                                entries rather than the old ORACLE.INI
 *                                format
 * 02 Jun 97                    . Made Oracle DLL runtime loaded rather
 *                                than statically linked.
 * 01 May 97  .017              . Changed create so CSTRING never creates
 *                                a CHAR
 *                              . Import imports short numerics as BYTE
 * 24 Mar 97  .016              . Fixed bug where last two characters of a
 *                                LONG where truncated on read.  This was
 *                                introduced when the buffer clearing code
 *                                was added.
 * 30 Nov 96  .015              . Fixed bug where NLSSORT was being called on
 *                                timestamp fields.
 * 13 Nov 96  .014              . Fixed bug where a view order that covers
 *                                fields in multiple files was generating the
 *                                wrong :n bind variable label
 * 06 Oct 96  .013              . Fixed bug where comparison components where
 *                                not bound properly for a view whose order
 *                                included non-primary file components.
 * 04 Oct 96  .012              . Fixed bug where if ADD set the NULL attribute
 *                                of a field, then a watched PUT straight after
 *                                would not use the NULL state.
 *                              . Added work around for the fact that UPDATE
 *                                does not work if the WHERE has a bound NULL
 *                                It only works if the WHERE uses IS NULL
 * 29 Oct 96  2.001.011         . Fixed bug where PUT would fail if the
 *                                record retrieved had NULL values
 * 28 Oct 96  2.001.010         . Fixed bug where BYTE was writting negative
 *                                values for number > 128
 * 26 Oct 96  2.001.009         . Added OEM support via NLSSORT().
 *                                Note:  This does not work for filters
 *  9 Oct 96  2.001.008         . Fixed bug where PROP:SQL would GPF if
 *                                the file had a timestamp structure in it
 *  9 Oct 96  2.001.007         . Fixed bug where OPEN(file) would GPF if
 *                                the table pointed at by file at been
 *                                previously opened by a different file
 *                                handle
 *  2 Oct 96  2.001.006         . Fixed bug where RESET(view) was not
 *                                setting the SET key.  So if the underlying
 *                                file was accessed by something else you
 *                                would get an Invalid Key error.
 * 28 Sep 96  2.001.005         . Moved position errors into log file
 *                              . Expanded position buffer logging to include
 *                              . All keys and all files
 * 27 Sep 96  2.001.004         . Added error messages to Position buffer
 *                                errors to tell developers what is wrong
 * 25 Sep 96  2.001.00 Build 3  . Merged traces into one file
 * 23 Sep 96  2.001.00 Build 2  . Changed import screen to use registry based
 *                                combo boxes
 *                              . Added importing of INDEXes and CONSTRAINTS
 * 23 Sep 96                    . Changed AddEqWhere to always add field even
 *                                if the field is NULL.  This is to avoid
 *                                the problem where the bind code is still
 *                                binding the field.  Also does not work on
 *                                REGET if the first time in the field is NULL
 *                                but subsequent times it is not.
 * 22 Apr 96  1.999.00 Build 2  . Fixed bug where dynamic index building was
 *                                erroring if there was a + in the list
 *                              . Fixed bug where Oracle LONG data types
 *                                did not work
 *                              . Fixed bug where RESET(key);NEXT(file)
 *                                did not return the correct record for a
 *                                non-unique key. (Introduced after 1.1)
 * 12 Apr 96  1.999.00 Build 1  . Built 2.0 version
 *                              . Mapped OCI error 9 to Duplicate Key rather
 *                                than 1 because ORA0001 == OCI9 ???
 * 24 Jan 96  Beta 1.1 Build 4  . Fixed DELETE after ADD so as it deletes the
 *                                added record and not the last fetched rec
 *                              . Use olog for 32bit connections
 *                              . Mapped 942 to FILNFD
 *                              . Added USEPRIMARY for view optimisation
 * 19 Jan 96                    . Fixed bug where a filter is not generated
 *                                correctly if the field is the first field
 *                                in a group
 * 18 Jan 96                    . Fixed CREATE INDEX to detect for the
 *                                correct error code for index already exists
 * 18 Jan 96  Beta 1.1 Build 3  . Added detection of empty string binding
 *                                on WHERE clauses and make it bind a space
 *                                This is an attempt to get SET and PUT to
 *                                work on empty CSTRING fields
 *                              . Added NULL testing on WHERE = code
 * 17 Jan 96                    . Added VIEW and SYNONYM imports
 * 12 Jan 96  Beta 1.1 Build 2  . Fix NULLs on ADD().  It was always using
 *                                the value set for first insert
 *                              . Clear NULL flag on PUT() and ADD() if
 *                                field contains data
 *                              . Fixed 1024 limit on SQL statements with
 *                                hints
 *                              . Added (null) printing to log file
 *                              . Added conditional commpile to allow
 *                                building of 1501 driver
 * 11 Jan 96  Beta 1.1 Build 1  . Optimise View not to add primary key
 *                                components onto unique keys
 *                              . Added asserts to trap for binding of
 *                                stack variables
 *  8 Jan 96  Release 1-K5      . Fixed NULL code so as not to bind stack
 *                                variable
 *  8 Jan 96  Release 1-K2
 *  6 Jan 96  Beta 0.2 Build 6  . Make sure NULL code works for Views
 *                              . Stop GPF when browsing file with no Owner
 *                              . Added extra code to try and get PUT to
 *                                work with Personal (with no success)
 *                              . Stop GPF when calling PROP:SQL before
 *                                opening a file
 *                              . Fixed PROP:SQL to work on non-SELECT
 *                                statements
 *                              . Fixed WATCH()/PUT() to work with Oracle
 *                                LONG data type
 * 29 Nov 95  Beta 0.2 Build 5  . Make sure field is blank when NULL retrieved
 * 14 Nov 95  Beta 0.2 Build 4  . Stop GPF if connect fails
 * 12 Nov 95  Beta 0.2 Build 3  . Fixed multi-level Joins
 *                              . Implemented PROP:Hint
 *                              . Added PROP:ConnectString
 *                              . Fixed bug in WATCH() for tables with a
 *                                field > 2000 chars
 *
 * After Beta 0.1               . Stop GPF on REGET() of blank position
 *                              . Add 32bit support
 *                              . Fixed NULL support
 *
 *****************************************************************************/
