/*----------------------------------------------------------
 * Built-in property constants for CDDW
 *
 * Property numbers less than PROPfirst are assumed to be referring
 * a numbered VBX control property
 *
 * Property numbers less than PROParrayfirst are single-valued,
 * and any index number supplied is decremented then added to the
 * property number before calling GetProperty or SetProperty
 *
 * Property numbers greater than or equal to PROParrayfirst are
 * array valued properties. Both the property number and the array
 * index are passed to Get/Set Property.
 *----------------------------------------------------------*/

#define PROPfirstproperty  0x7000  // values below this reserved for custom controls

// 7A00-7AFF more standard properties
// 7B00-7BFF printer properties
// 7C00-7CFF standard properties
// 7D00-7DFF array valued properties

#define PROPtext           0x7C00  // single value (parameter to control itself)
#define PROPtype           0X7C01  // single value (CREATE:xxxx)
#define PROPat             0x7C02  // array valued (4 values)
#define PROPxpos           0x7C02  // single value, equivalent to PROPat[1]
#define PROPypos           0x7C03  // single value, equivalent to PROPat[2]
#define PROPwidth          0x7C04  // single value, equivalent to PROPat[3]
#define PROPheight         0x7C05  // single value, equivalent to PROPat[4]

#define PROPcenter         0x7C06  // 0 = off, else on
#define PROPcenteroffset   0x7C07  // single value, equivalent to PROPcenter[2]
#define PROPleft           0x7C08  // 0 = off, else on
#define PROPleftoffset     0x7C09  // single value, equivalent to PROPleft[2]
#define PROPdecimal        0x7C0A  // 0 = off, else on
#define PROPdecimaloffset  0x7C0B  // single value, equivalent to PROPdecimal[2]
#define PROPright          0x7C0C  // 0 = off, else on
#define PROPrightoffset    0x7C0D  // single value, equivalent to PROPright[2]

#define PROPabove          PROPdecimal  // Aliases for sheet controls.
#define PROPbelow          PROPcenter

#define PROPclass          0x7C0E  // array valued (2 values)
#define PROPvbxfile        0x7C0E  // single value, equivalent to PROPclass[1]
#define PROPvbxname        0x7C0F  // single value, equivalent to PROPclass[2]

#define PROPfont           0x7C10  // array valued (4 values)
#define PROPfontname       0x7C10  // single value, equivalent to PROPfont[1]
#define PROPfontsize       0x7C11  // single value, equivalent to PROPfont[2]
#define PROPfontcolor      0x7C12  // single value, equivalent to PROPfont[3]
#define PROPfontstyle      0x7C13  // single value, equivalent to PROPfont[4]

#define PROPrange          0x7C14  // array valued (2 values)
#define PROPrangelow       0x7C14  // single value, equivalent to PROPrange[1]
#define PROPrangehigh      0x7C15  // single value, equivalent to PROPrange[2]

#define PROPvcr            0x7C16  // 0 = off, else on
#define PROPvcrfeq         0x7C17  // blank, feq, or picture, = PROPvcr[2]

#define PROPdouble         0x7C19  // 0 = off, else on
#define PROPresize         0x7C1A  // 0 = off, else on
#define PROPnoframe        0x7C1B  // 0 = off, else on

#define PROPmm             0x7C1C  // 0 = off, else on
#define PROPthous          0x7C1D  // 0 = off, else on
#define PROPpoints         0x7C1E  // 0 = off, else on

#define PROPsum            0x7C40  // 0 = off, else on
#define PROPave            0x7C41  // 0 = off, else on
#define PROPmax            0x7C42  // 0 = off, else on
#define PROPmin            0x7C43  // 0 = off, else on
#define PROPcnt            0x7C44  // 0 = off, else on
#define PROPpageno         0x7C45  // 0 = off, else on
#define PROPpage           0x7C50  // 0 = off, else on


#define PROPabsolute       0x7C51  // 0 = off, else on
#define PROPalone          0x7C52  // 0 = off, else on
#define PROPpreview        0x7C53  // single queue or queuefield (write-only)
#define PROPflushpreview   0x7C54  // 0 = off, else on (always 0 at open)
#define PROPfirst          0x7C55  // 0 = off, else on
#define PROPlast           0x7C56  // 0 = off, else on
#define PROPins            0x7C57  // 0 = off, else on
#define PROPovr            0x7C58  // 0 = off, else on
#define PROPboxed          0x7C59  // 0 = off, else on
#define PROPcap            0x7C5A  // 0 = off, else on
// spare #define PROPcolor          0x7C5B  // single value, COLORnone if none
#define PROPcolumn         0x7C5C  // 0 = off, else column number
#define PROPcursor         0x7C5D  // single value, empty string if not specifed
#define PROPdefault        0x7C5E  // 0 = off, else on
#define PROPdisable        0x7C5F  // 0 = off, else on
#define PROPdrop           0x7C60  // single value, 0 if none (can't change to or from 0)
#define PROPfill           0x7C61  // single value, COLORnone if none
#define PROPfrom           0x7C62  // single queue, queuefield or string (write-only)
#define PROPfull           0x7C63  // 0 = off, else on
#define PROPgray           0x7C64  // 0 = off, else on
#define PROPhide           0x7C65  // 0 = off, else on
#define PROPhlp            0x7C66  // single value
#define PROPhscroll        0x7C67  // 0 = off, else on
#define PROPicon           0x7C68  // single value, empty string if not specified
#define PROPiconize        0x7C69  // 0 = off, else on
#define PROPimm            0x7C6A  // 0 = off, else on
#define PROPkey            0x7C6B  // single value
#define PROPlandscape      0x7C6C  // 0 = off, else on
#define PROPmark           0x7C6D  // single queue, queuefield or array (write-only)
#define PROPmask           0x7C6E  // 0 = off, else on
#define PROPmaximize       0x7C6F  // 0 = off, else on
#define PROPmdi            0x7C70  // 0 = off, else on (read only)
#define PROPmeta           0x7C71  // 0 = off, else on
#define PROPmodal          0x7C72  // 0 = off, else on (read only)
#define PROPmsg            0x7C73  // single value
#define PROPnobar          0x7C74  // 0 = off, else on
#define PROPnomerge        0x7C75  // 0 = off, else on
#define PROPpageafter      0x7C77   // 0 = off, else on
#define PROPpageafternum   0x7C77   // single value, 0 if absent == pageafter[2]
#define PROPpagebefore     0x7C78   // 0 = off, else on
#define PROPpagebeforenum  0x7C79   // single value, 0 if absent == pagebefore[2]
#define PROPpassword       0x7C7A   // 0 = off, else on
#define PROPreadonly       0x7C7B   // 0 = off, else on
#define PROPreq            0x7C7C   // 0 = off, else on
#define PROPreset          0x7C7D   // single value (nesting depth, or 0)
#define PROPround          0x7C7E   // 0 = off, else on
#define PROPscroll         0x7C7F   // 0 = off, else on
#define PROPseparate       0x7C80   // 0 = off, else on
#define PROPskip           0x7C81   // 0 = off, else on
#define PROPstd            0x7C82   // single value
#define PROPstep           0x7C83   // single value
#define PROPsystem         0x7C84   // 0 = off, else on
#define PROPformat         0x7C85   // single value
#define PROPtimer          0x7C86   // single value, 0 if none
#define PROPcheck          0x7C87   // 0 = off, else on
#define PROPtrn            0x7C88   // 0 = off, else on
#define PROPupr            0x7C89   // 0 = off, else on
#define PROPolduse         0x7C8A   // single variable value, write only
#define PROPvscroll        0x7C8B   // 0 = off, else on
#define PROPwithnext       0x7C8C   // single value, 0 if absent
#define PROPwithprior      0x7C8D   // single value, 0 if absent
#define PROPitems          0x7C8E   // single value (read only, runtime only)
#define PROPselstart       0x7C8F   // single value (runtime only)
#define PROPselend         0x7C90   // single value (runtime only)
#define PROPauto           0x7C91   // 0 = off, else on (runtime only for now)
#define PROPtoolbox        0x7C92   // 0 = off, else on
#define PROPpalette        0x7C93   // single value
#define PROPimagebits      0x7C94   // string variable (runtime only)
#define PROPthread         0x7C95   // read only, runtime only
#define PROPhandle         0x7C96   // read only, runtime only
#define PROPactive         0x7C97   // runtime only
#define PROPvbxevent       0x7C98   // read only, runtime only
#define PROPfollows        0x7C99   // write only, runtime only
#define PROPacceptall      0x7C9A   // 0 = off, else on (runtime only, also set by SELECT() and cleared by SELECT(n))
#define PROPtouched        0x7C9B   // 0 = off, else on (runtime only)
#define PROPappmodal       0x7C9D   // 0 = off, else on (runtime only, currently)
#define PROPvscrollpos     0x7C9E   // Scroll thumb position (0-100)
#define PROPclipbits       0x7C9F   // 1 => Clip image
#define PROPtooltip        0x7CA0   // Tool tip text
#define PROPbarpercent     0x7CA1   // Show percent on the bar
#define PROPbarposition    0x7CA2   // Show position on the bar
#define PROPvertical       0x7CA3   // Is slider vertical
#define PROPticks          0x7CA4   // 0 = off, else frequency
// spare #define PROPvalue          0x7CA5   // value for a radio to assign to use.
#define PROPlfnsupport     0x7CA5   // use long filenames (where allowed)
#define PROPprogress       0x7CA6   // progress bar position (runtime only)
#define PROPvisible        0x7CA7   // is control visible (read-only, runtime-only)
#define PROPenabled        0x7CA8   // is control enabled (read-only, runtime-only)
#define PROPwizard         0x7CA9   // wizard style sheets
#define PROPchoicefeq      0x7CAA   // currect field equate of a choice in sheet, option
#define PROPclienthandle   0x7CAB   // client window handle
#define PROPlinecount      0x7CAC   // lines of text in a TEXT control (read-only, runtime only)
#define PROPminwidth       0x7CAD   // window's minimum width (runtime only)
#define PROPminheight      0x7CAE   // window's minimum height (runtime only)
#define PROPmaxwidth       0x7CAF   // window's maximum width (runtime only)
#define PROPmaxheight      0x7CB0   // window's maximum height (runtime only)
#define PROPnotips         0x7CB1   // Disable tooltips on system/window/control (runtime only)
#define PROPtipdelay       0x7CB2   // Delay before tips appear (16-bit runtime only, system only)
#define PROPtipdisplay     0x7CB3   // Time for which tips are displayed (16-bit runtime only, system only)
#define PROPwndproc        0x7CB4   // Window procedure set/get (for PROP:handle)
#define PROPclientwndproc  0x7CB5   // Window procedure set/get (for PROP:clienthandle)
#define PROPimageblob      0x7CB6   // BLOB handle (runtime only)
#define PROPspread         0x7CB7   // SPREAD attribute
#define PROPscreentext     0x7CBA   // may differ from PROP:text for eg ENTRY
#define PROPhscrollpos     0x7CBB   // Horizontal scroll position
#define PROPlazydisplay    0x7CBC   // Don't force update immediately
#define PROPdefermove      0x7CBD   // Delay window positioning (system property)
#define PROPappinstance    0x7CBE   // HINSTANCE of exe file (read only)
#define PROPvalue          0x7CBF   // For a radio control or tab
#define PROPtruevalue   PROPvalue   // For a check control
#define PROPfalsevalue     0x7CC0   // For a check control
#define PROPDDEtimeout     0x7CC1   // DDE timeout
#define PROPtabrows        0x7CC2   // Num rows in a tab (read only)
#define PROPnumtabs        0x7CC3   // Num tabs owned by a sheet (read only)
#define PROPlibversion     0x7CC4   // runtime library version
#define PROPexeversion     0x7CC5   // executable file version

#define PROPcreate          0x7CC6  // create new ole object
#define PROPsaveas          0x7CC7
#define PROPopen            0x7CC8
#define PROPblob            0x7CC9
#define PROPdoverb          0x7CCA  // activate Ole Object
#define PROPsizemode        0x7CCB  // controls the way not runnung object is displayed
#define PROPselectinterface 0x7CCC  // Select currnet ole interface
#define PROPaddref          0x7CCD  // Increment referenc count
#define PROPrelease         0x7CCE  // Decrement referenc count
#define PROPdeactivate      0x7CCF  // deactivate object
#define PROPupdate          0x7CD0  // update object
#define PROPpaste           0x7CD1  // embed object from clipboard
#define PROPreportexception 0x7CD2  // pop up ole exceptions
#define PROPpastelink       0x7CD3  // create link from clipboard
#define PROPcopy            0x7CD4  // copy object to clipboard
#define PROPcanpaste        0x7CD5
#define PROPcanpastelink    0x7CD6
#define PROPwindowui        0x7CD7
#define PROPdesignmode      0x7CD8  // ocx designemode
#define PROPctrl            0x7CD9  // is ocx control
#define PROPgrabhandles     0x7CDA
#define PROPole             0x7CDB  // is ole  or ocx object ?
#define PROPisradio         0x7CDC  // is it a radio button
#define PROPlasteventname   0x7CDD
#define PROPclsid           0x7CDE
#define PROPprogid          0x7CDF

#define PROPbevelstyle      0x7CE0   // kind of bevel for boxes
#define PROPjoinupdown      0x7CE2   // Place < > buttons in same place
#define PROPnosheet         0x7CE3   // Just display the tabs - no sheet
#define PROPbevelouter      0x7CE4   // Size, direction of outer bevel
#define PROPbevelinner      0x7CE5   // Size, direction of inner bevel
#define PROPlinewidth       0x7CE7   // Pen width for a BOX or LINE
#define PROPangle           0x7CE9   // Angle of text for a string control

#define PROPautosize        0x7CEA  //
#define PROPclip            0x7CEB  //
#define PROPstretch         0x7CEC  //
#define PROPzoom            0x7CED  //
#define PROPcompatibility   0x7CEE  //
#define PROPdesign          0x7CEF  //
#define PROPdocument        0x7CF0  // create ole object from file (like x.doc i.e. a word document)
#define PROPlink            0x7CF1  // as above but link
#define PROPcreatefromfile    PROPdocument
#define PROPcreatelinktofile  PROPlink
#define PROPalign           0x7CF2 // vb ocx align
#define PROPcancel          0x7CF3 // vb ocx cancel
#define PROPtextalign       0x7CF4 // vb ocx ambient property 0-number right text left, 1-left, 2-center, 3-right 4-sustify
#define PROPobject          0x7CF5
#define PROPlicense         0x7CF6
#define PROPignorethreading 0x7CF7
#define PROPlanguage        0x7CF8

#define PROPcolor           0x7CFA // up to three components (see below)
#define PROPfillcolor       PROPcolor
#define PROPselectedcolor   0x7CFB // single value, COLORnone if none
#define PROPselectedfillcolor 0x7CFC // single value, COLORnone if none
#define PROPlineheight      0x7CFD // Height of a line in a list box
#define PROPdropwidth       0x7CFE // Width of a drop box
#define PROPalwaysdrop      0x7CFF // Drop control always displays the list on selection change

#define PROPup              0x7A00 // Different orientations of text in a sheet
#define PROPdown            0x7A01 // control
#define PROPupsidedown      0x7A02
#define PROPheaderheight    0x7A03 // Height of the header on a list box
#define PROPchecked         0x7A04 // Is a check box set?
#define PROPautopaper       0x7A05 // Report engine automatically picks best paper size
#define PROPsingle          0x7A06 // Single line text box (for Hebrew/Arabic support)
#define PROPparent          0x7A07 // Field equate of parent control
#define PROPrepeat          0x7A08 // Repeat rate of IMM button
#define PROPeventswaiting   0x7A09 // Does this window have any further events pending?
#define PROPddemode         0x7A0A // 0 = normal, 1 = events to top window of thread, other values reserved
#define PROPthreading       0x7A0B // Set to 0 to disable all threading in APP (forces SDI behaviour)
#define PROPtempnamefunc    0x7A0C // function to call to get metafile names
#define PROPmarkptr         0x7A0D // Internal hook for internet
#define PROPfromptr         0x7A0E // Internal hook for internet
#define PROPdelay           0x7A0F // Delay before IMM button repeats
#define PROPuse             0x7A10 // Array valued: ! PROP:use or PROP:use,1 sets use variable
#define PROPfeq             0x7A11 // PROP:feq or PROP:use,2 sets feq number
#define PROPlistfeq         0x7A12 // PROP:listfeq or PROP:use,3 sets feq of list part of combo/drop
#define PROPbuttonfeq       0x7A13 // PROP:buttonfeq or PROP:use,4 sets feq of button part of combo/drop
#define PROPflat            0x7A14 // Explorer 3.0 style buttons

#define PROPwallpaper       0x7A15 // Background image of a window
#define PROPtiled           0x7A16 // Should we tile background image?
#define PROPcentered        0x7A17 // Should we center background image?
#define PROPchildindex      0x7A18 // Reverse of PROPchild,nn
#define PROPintoolbar       0x7A19 // Is this on the toolbar?
#define PROPprintmode       0x7A1A // How do images print?
#define PROPrejectcode      0x7A1B // Was text in entry/spin valid - if not why not
#define PROPvlbproc         0x7A1C // "Virtual list box" proc address
#define PROPvlbval          0x7A1D // "Virtual list box" proc user value
#define PROPtempimage       0x7A1E // Dumps image into temp file, returns name (read only)
#define PROPtempimagestatus 0x7A1F // Was the last temp image a new creation?

#define PROPtemppath        0x7A20 // Array valued (2 defined so far)
#define PROPtemppagepath    0x7A20 // Equivalent to {PROP:temppath, 1}
#define PROPtempimagepath   0x7A21 // Equivalent to {PROP:temppath, 2}

#define PROPlibhook         0x7A30 // Array valued (9 defined so far)
#define PROPcolordialoghook 0x7A30 // Equivalent to {PROP:libhook, 1}
#define PROPfiledialoghook  0x7A31 // Equivalent to {PROP:libhook, 2}
#define PROPfontdialoghook  0x7A32 // Equivalent to {PROP:libhook, 3}
#define PROPprinterdialoghook 0x7A33 // Equivalent to {PROP:libhook, 4}
#define PROPhalthook        0x7A34 // Equivalent to {PROP:libhook, 5}
#define PROPmessagehook     0x7A35 // Equivalent to {PROP:libhook, 6}
#define PROPstophook        0x7A36 // Equivalent to {PROP:libhook, 7}
#define PROPasserthook      0x7A37 // Equivalent to {PROP:libhook, 8}
#define PROPfatalerrorhook  0x7A38 // Equivalent to {PROP:libhook, 9}

#define PROPnowidth         0x7A40 // was a width specified?
#define PROPnoheight        0x7A41 // was a height specified?
#define PROPnextpageno      0x7A42 // next page number in report

#define PROParrayfirst     0x7D00
#define PROPstatus         0x7D00   // array valued
#define PROPstatustext     0x7D01   // array valued
#define PROPdragid         0x7D02   // array valued
#define PROPdropid         0x7D03   // array valued
#define PROPvbxeventarg    0x7D04   // array valued
#define PROPedit           0x7D05   // array valued
#define PROPalrt           0x7D06   // array valued
#define PROPiconlist       0x7D07   // array valued
#define PROPline           0x7D08   // array valued, TEXT control line contents (read-only, runtime only)
#define PROPtally          0x7D09   // array valued, tally attribut on report string control
#define PROPchild          0x7D0A   // array valued, iterate through immediate children
#define PROPnextfield      0x7D0B   // array valued, iterate through all children of a window
                                    // NB order is NOT defined.
