/*----------------------------------------------------------------
 * WSLGLOBAL.H - included by all wsl source files
 * Copyright (C) 1993 Clarion Software Corporation
 * All rights reserved
 *----------------------------------------------------------------*/

#ifndef WSLGLOBAL_INCL
#define WSLGLOBAL_INCL
#define STRICT
#include <clwclib.h>
#include <cla.h>
#include <windows.h>
#include <WslHash.hpp>

#ifdef __cplusplus
extern "C" {
#endif

#pragma save, name(prefix=>"")

typedef LRESULT (CALLBACK *WINCALLBACK)(HWND,UINT,WPARAM,LPARAM);
typedef void (*THREADPROC)(void);

extern BOOL       Wsl@LibraryAlive;     /* FALSE during startup/shutdown      */
extern BOOL       Wsl@ErrorTerm;        /* Set true if we have a fatal error  */
extern BOOL       Wsl@Use3d;            /* 3d-look for newly created controls */
extern HINSTANCE  Wsl@LibInstance;      /* Library instance                   */
extern HINSTANCE  Wsl@AppInstance;      /* Application (EXE) instance         */
extern HINSTANCE  Wsl@ResourceInstance; /* Current resource file instance     */
extern HWND       Wsl@hClient;          /* MDI Client window handle           */
extern BOOL       Wsl@CloseDownAborted; /* MDI children refused to close      */
extern HWND       Wsl@hFrame;           /* MDI Frame window handle            */
extern THREADPROC Wsl@IdleProc;         /* Points to IDLE procedure           */
extern HWND       Wsl@LastGoodFocus;    /* Last thing which had focus to keep */
extern HCURSOR    Wsl@UserCursor;       /* User has supplied a cursor         */
extern BOOL       Wsl@LFNEnabled;       /* Should use long filenames if possible if TRUE */

void Wsl$FatalError(int, const char *);

/* Event numbers */

#define EVENT_FIRST       (WM_USER+0x1000)

/* Field-action events (ACCEPTED() returns field number) */

#define EVENT_ACCEPTED     (EVENT_FIRST+0x01)
#define EVENT_NEWSELECTION (EVENT_FIRST+0x02)
#define EVENT_SCROLLUP     (EVENT_FIRST+0x03)
#define EVENT_SCROLLDOWN   (EVENT_FIRST+0x04)
#define EVENT_PAGEUP       (EVENT_FIRST+0x05)
#define EVENT_PAGEDOWN     (EVENT_FIRST+0x06)
#define EVENT_SCROLLTOP    (EVENT_FIRST+0x07)
#define EVENT_SCROLLBOTTOM (EVENT_FIRST+0x08)
#define EVENT_LOCATE       (EVENT_FIRST+0x09)

#define EVENT_MOUSEDOWN    (EVENT_ACCEPTED)
#define EVENT_MOUSEUP      (EVENT_FIRST+0x0a)
#define EVENT_MOUSEIN      (EVENT_FIRST+0x0b)
#define EVENT_MOUSEOUT     (EVENT_FIRST+0x0c)
#define EVENT_MOUSEMOVE    (EVENT_FIRST+0x0d)
#define EVENT_VBXEVENT     (EVENT_FIRST+0x0e)
#define EVENT_ALERTKEY     (EVENT_FIRST+0x0f)
#define EVENT_PREALERT     (EVENT_FIRST+0x10)
#define EVENT_DRAGGING     (EVENT_FIRST+0x11)
#define EVENT_DRAG         (EVENT_FIRST+0x12)
#define EVENT_DROP         (EVENT_FIRST+0x13)
#define EVENT_SCROLLDRAG   (EVENT_FIRST+0x14)
#define EVENT_TABCHANGING  (EVENT_FIRST+0x15)
#define EVENT_EXPANDING    (EVENT_FIRST+0x16)
#define EVENT_CONTRACTING  (EVENT_FIRST+0x17)
#define EVENT_EXPANDED     (EVENT_FIRST+0x18)
#define EVENT_CONTRACTED   (EVENT_FIRST+0x19)
#define EVENT_REJECTED     (EVENT_FIRST+0x1A)
#define EVENT_DROPPINGDOWN (EVENT_FIRST+0x1B)
#define EVENT_DROPPEDDOWN  (EVENT_FIRST+0x1C)
#define EVENT_SCROLLTRACK  (EVENT_FIRST+0x1D)
#define EVENT_COLUMNRESIZE (EVENT_FIRST+0x1E)

/* Field-selection events (SELECTED() returns field number) */

#define EVENT_SELECTED    (EVENT_FIRST+0x101)

/* Field-independent events (ACCEPTED()/SELECTED() return 0) */

#define EVENT_ANONFIRST   (EVENT_FIRST+0x200)
#define EVENT_CLOSEWINDOW (EVENT_FIRST+0x201)
#define EVENT_CLOSEDOWN   (EVENT_FIRST+0x202)
#define EVENT_OPENWINDOW  (EVENT_FIRST+0x203)
#define EVENT_OPENFAILED  (EVENT_FIRST+0x204)
#define EVENT_LOSEFOCUS   (EVENT_FIRST+0x205)
#define EVENT_GAINFOCUS   (EVENT_FIRST+0x206)

#define EVENT_SUSPEND     (EVENT_FIRST+0x208)
#define EVENT_RESUME      (EVENT_FIRST+0x209)
#define EVENT_NOTIFY      (EVENT_FIRST+0x20A)
#define EVENT_TIMER       (EVENT_FIRST+0x20B)

#define EVENT_DDEFIRST    (EVENT_FIRST+0x20C)
#define EVENT_DDEREQUEST  (EVENT_FIRST+0x20C)
#define EVENT_DDEADVISE   (EVENT_FIRST+0x20D)
#define EVENT_DDEDATA     (EVENT_FIRST+0x20E)
#define EVENT_DDEEXECUTE  (EVENT_FIRST+0x20F)
#define EVENT_DDEPOKE     (EVENT_FIRST+0x210)
#define EVENT_DDECLOSED   (EVENT_FIRST+0x211)
#define EVENT_DDELAST     (EVENT_FIRST+0x211)

#define EVENT_MOVE        (EVENT_FIRST+0x220)
#define EVENT_SIZE        (EVENT_FIRST+0x221)
#define EVENT_RESTORE     (EVENT_FIRST+0x222)
#define EVENT_MAXIMIZE    (EVENT_FIRST+0x223)
#define EVENT_ICONIZE     (EVENT_FIRST+0x224)
#define EVENT_COMPLETED   (EVENT_FIRST+0x225)
#define EVENT_MOVED       (EVENT_FIRST+0x230)
#define EVENT_SIZED       (EVENT_FIRST+0x231)
#define EVENT_RESTORED    (EVENT_FIRST+0x232)
#define EVENT_MAXIMIZED   (EVENT_FIRST+0x233)
#define EVENT_ICONIZED    (EVENT_FIRST+0x234)

#define EVENT_BUILDFILE   (EVENT_FIRST+0x240)
#define EVENT_BUILDKEY    (EVENT_FIRST+0x241)
#define EVENT_BUILDDONE   (EVENT_FIRST+0x242)

#define EVENT_USER        (EVENT_FIRST+0x400)
#define EVENT_LAST        (EVENT_FIRST+0x0fff)

/*-----------------------------------------------------------
 * Internal message numbers
 *-----------------------------------------------------------*/

#define QUERY_SETUSE      (EVENT_FIRST+0x1001)
#define QUERY_SELECTED    (EVENT_FIRST+0x1002)
#define QUERY_COMMAND     (EVENT_FIRST+0x1003)
#define QUERY_HOTCHANGE   (EVENT_FIRST+0x1004)
#define QUERY_HELP        (EVENT_FIRST+0x1005)
#define QUERY_QUIT        (EVENT_FIRST+0x1006)
#define QUERY_REPLY       (EVENT_FIRST+0x1007)
#define QUERY_LOSEFOCUS   (EVENT_FIRST+0x1008)
#define QUERY_PSCREEN     (EVENT_FIRST+0x1009)
#define QUERY_PFIELD      (EVENT_FIRST+0x100a)

#define QUERY_COLOR       (EVENT_FIRST+0x100c)
#define QUERY_FINDINLIST  (EVENT_FIRST+0x100d)
#define QUERY_HSCROLL     (EVENT_FIRST+0x100e)

#define QUERY_SETFOCUS    (EVENT_FIRST+0x100f)
#define QUERY_NOTIFY      (EVENT_FIRST+0x1010)

/** list/combo box events **/
#define QUERY_NEWSELECTION (EVENT_FIRST+0x1011)
#define QUERY_LOCATE       (EVENT_FIRST+0x1012)

#define QUERY_CLIPTOWINDOW (EVENT_FIRST+0x1013)
#define QUERY_STARTTHREAD  (EVENT_FIRST+0x1014)
#define QUERY_DEBUGSWITCH  (EVENT_FIRST+0x1015)  // Don't renumber (used by debugger)
#define QUERY_TIMER        (EVENT_FIRST+0x1016)
#define QUERY_DRAG         (EVENT_FIRST+0x1017)
#define QUERY_DRAGOVER     (EVENT_FIRST+0x1018)
#define QUERY_DRAGMOVE     (EVENT_FIRST+0x1019)
#define QUERY_DRAGSIG      (EVENT_FIRST+0x101A)
#define QUERY_EVENT        (EVENT_FIRST+0x101B)
#define QUERY_CONTINUETHREAD (EVENT_FIRST+0x101C)
#define QUERY_EDITDONE     (EVENT_FIRST+0x101D)
#define QUERY_NOCLIPWINDOW (EVENT_FIRST+0x101E)
#define QUERY_NOCLIPBACK   (EVENT_FIRST+0x101F)
#define QUERY_BUILDFILE    (EVENT_FIRST+0x1020)
#define LASTCLAMESSAGE     (EVENT_FIRST+0x1fff)

/* Event loop responses */
#define LOOPBREAK  -1
#define LOOPFALL   0
#define LOOPCYCLE  1


#define FIELDBASE       0x4000

#pragma restore
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
class field;
class menubar;
class toolbar;
class reportscreen;
class screen;
class UseDesc;
class control;
class groupcontrol;

#ifdef __32BIT__
#define SCREENPOOL
#else
extern  UINT screenpool;
#define SCREENPOOL <screenpool>
#endif

typedef screen SCREENPOOL *PSCREEN;
typedef reportscreen SCREENPOOL *PREPSCREEN;

typedef unsigned long PCONV;

control *GetWindowControlPtr(HWND);

#endif



#endif //WSLGLOBAL_INCL

