/******************************************************************************/
/*                                  _OSIO.H                                   */
/*                                                                            */
/*  This header defines general functions and macros used to invoke low-level */
/*  Operating System functions.                                               */
/*                                                                            */
/*  Sean Wilson, 06 December 1994                                             */
/******************************************************************************/
// NH added _OsFileDate and _OsFileTrucate
#ifndef _OSIO_H
#define _OSIO_H

#if defined(__cplusplus)
extern "C" {
#endif

/*** MS-DOS Only functions making use of inline functions defined in _dos.h ***/
#if defined(__MSDOS__) || defined(_XTD) || (defined(_WIN) && !defined(__32BIT__))
#include "_dos.h"


/*** File handle type                                                       ***/
#define _OSHFILE  int


/*** Error Codes                                                            ***/
#define _OSEBADFNCODE                   _DOSEBADFNCODE
#define _OSENOENT                       _DOSENOENT
#define _OSENOPATH                      _DOSENOPATH
#define _OSENOHND                       _DOSENOHND
#define _OSEACCESS                      _DOSEACCESS
#define _OSEBADHND                      _DOSEBADHND
#define _OSECORMEM                      _DOSECORMEM
#define _OSENOMEM                       _DOSENOMEM
#define _OSEBADBLK                      _DOSEBADBLK
#define _OSEBADENV                      _DOSEBADENV
#define _OSEBADFMT                      _DOSEBADFMT
#define _OSEBADACC                      _DOSEBADACC
#define _OSEBADDAT                      _DOSEBADDAT
#define _OSEBADDRV                      _DOSEBADDRV
#define _OSECURDIR                      _DOSECURDIR
#define _OSEBADDEV                      _DOSEBADDEV
#define _OSENOFILE                      _DOSENOFILE
#define _OSEWPROT                       _DOSEWPROT
#define _OSEBADDSK                      _DOSEBADDSK
#define _OSENOTREADY                    _DOSENOTREADY
#define _OSEBADCMD                      _DOSEBADCMD
#define _OSEBADCRC                      _DOSEBADCRC
#define _OSEBADLEN                      _DOSEBADLEN
#define _OSESEEK                        _DOSESEEK
#define _OSBADMEDIA                     _DOSBADMEDIA
#define _OSEBADSECT                     _DOSEBADSECT
#define _OSENOPAPER                     _DOSENOPAPER
#define _OSEWRFAULT                     _DOSEWRFAULT
#define _OSERDFAULT                     _DOSERDFAULT
#define _OSEGENFAIL                     _DOSEGENFAIL
#define _OSESHRVIOL                     _DOSESHRVIOL
#define _OSELCKVIOL                     _DOSELCKVIOL
#define _OSEBADCHG                      _DOSEBADCHG
#define _OSENOFCB                       _DOSENOFCB
#define _OSESHOFLO                      _DOSESHOFLO


/*** Origin for _OsSeekFile()                                               ***/
#define _OS_SEEKSET                     _DOS_SEEKSET
#define _OS_SEEKCUR                     _DOS_SEEKCUR
#define _OS_SEEKEND                     _DOS_SEEKEND


/*** File attributes for _OsCreateFile()                                    ***/
#define _OS_FANORMAL                    _DOS_FANORMAL
#define _OS_FAREADONLY                  _DOS_FAREADONLY
#define _OS_FAHIDDEN                    _DOS_FAHIDDEN
#define _OS_FASYSTEM                    _DOS_FASYSTEM


/*** File attributes for _OsOpenFile()                                      ***/
#define _OS_OAREADONLY                  _DOS_OAREADONLY
#define _OS_OAWRITEONLY                 _DOS_OAWRITEONLY
#define _OS_OAREADWRITE                 _DOS_OAREADWRITE


/*** Permission flags for _OsOpenFile()                                     ***/
#define _OS_SHCOMPAT                    _DOS_SHCOMPAT
#define _OS_SHDENYRW                    _DOS_SHDENYRW
#define _OS_SHDENYWR                    _DOS_SHDENYWR
#define _OS_SHDENYRD                    _DOS_SHDENYRD
#define _OS_SHDENYNO                    _DOS_SHDENYNO

#define _OS_NOINHERIT                   _DOS_NOINHERIT

#if defined(_WIN)
int _OsCreateFile(const char *path, _OSHFILE *h, unsigned attr);
int _OsDeleteFile(const char *path);
int _OsOpenFile(const char *path, _OSHFILE *h, unsigned attr);
int _OsRenameFile(const char * OldNameAnsi, const char * NewNameAnsi);
int _OsMakeDirectory(const char *PathAnsi);
int _OsRemoveDirectory(const char *PathAnsi);
int _OsChangeDirectory(const char *PathAnsi);
int _OsGetCurrentDir(int Drv, char *pDir/*[256]*/);
int _OsGetFileMode(const char *Path, int *pMode);
int _OsFileExists(const char * FileNameAnsi);
#else
#define _OsCreateFile(path,ph,attr)     _DosCreateFile(path,ph,attr)
#define _OsOpenFile(path,ph,attr)       _DosOpenFile(path,ph,attr)
#define _OsDeleteFile(path)             _DosDeleteFile(path)
#endif

#define _OsCloseFile(h)                 _DosCloseFile(h)
#define _OsCommitFile(h)                _DosCommitFile(h)
#define _OsGetFileLen(h,plen)           _DosGetFileLen(h,plen)
#define _OsGetFilePos(h,ppos)           _DosGetFilePos(h,ppos)
#define _OsLockFile(h,pos,len)          _DosLockFile(h,pos,len)
#define _OsReadFile(h,len,buf,plen)     _DosReadFile(h,len,buf,plen)
#define _OsSeekFile(h,ofs,org)          _DosSeekFile(org,h,ofs)
#define _OsUnlockFile(h,pos,len)        _DosUnlockFile(h,pos,len)
#define _OsWriteFile(h,len,buf,plen)    _DosWriteFile(h,len,buf,plen)
#define _OsFileDate(h,pdatetime)        _DosFileDate(h,pdatetime)
#define _OsSetFileDate(h, datetime)     _DosSetFileDate(h,datetime)
#define _OsFileTruncate(h)              _DosFileTruncate(h)

/*** OS/2 Only functions calling into the OS/2 API                          ***/
#elif defined(__OS2__)

#define INCL_DOS
#define INCL_DOSERRORS

#define _DRIVER_NAME    DRIVER_NAME
#undef DRIVER_NAME
#include "os2def.h"
#include "bse.h"
#undef DRIVER_NAME
#define DRIVER_NAME     _DRIVER_NAME


/*** File handle type                                                       ***/
#define _OSHFILE  HFILE


/*** Error Codes                                                            ***/
#define _OSEBADFNCODE                   ERROR_INVALID_FUNCTION
#define _OSENOENT                       ERROR_FILE_NOT_FOUND
#define _OSENOPATH                      ERROR_PATH_NOT_FOUND
#define _OSENOHND                       ERROR_TOO_MANY_OPEN_FILES
#define _OSEACCESS                      ERROR_ACCESS_DENIED
#define _OSEBADHND                      ERROR_INVALID_HANDLE
#define _OSECORMEM                      ERROR_ARENA_TRASHED
#define _OSENOMEM                       ERROR_NOT_ENOUGH_MEMORY
#define _OSEBADBLK                      ERROR_INVALID_BLOCK
#define _OSEBADENV                      ERROR_BAD_ENVIRONMENT
#define _OSEBADFMT                      ERROR_BAD_FORMAT
#define _OSEBADACC                      ERROR_INVALID_ACCESS
#define _OSEBADDAT                      ERROR_INVALID_DATA
#define _OSEBADDRV                      ERROR_INVALID_DRIVE
#define _OSECURDIR                      ERROR_CURRENT_DIRECTORY
#define _OSEBADDEV                      ERROR_NOT_SAME_DEVICE
#define _OSENOFILE                      ERROR_NO_MORE_FILES
#define _OSEWPROT                       ERROR_WRITE_PROTECT
#define _OSEBADDSK                      ERROR_I24_DEVICE_IN_USE
#define _OSENOTREADY                    ERROR_NOT_READY
#define _OSEBADCMD                      ERROR_BAD_COMMAND
#define _OSEBADCRC                      ERROR_CRC
#define _OSEBADLEN                      ERROR_BAD_LENGTH
#define _OSESEEK                        ERROR_SEEK
#define _OSBADMEDIA                     ERROR_NOT_DOS_DISK
#define _OSEBADSECT                     ERROR_SECTOR_NOT_FOUND
#define _OSENOPAPER                     ERROR_OUT_OF_PAPER
#define _OSEWRFAULT                     ERROR_WRITE_FAULT
#define _OSERDFAULT                     ERROR_READ_FAULT
#define _OSEGENFAIL                     ERROR_GEN_FAILURE
#define _OSESHRVIOL                     ERROR_SHARING_VIOLATION
#define _OSELCKVIOL                     ERROR_LOCK_VIOLATION
#define _OSEBADCHG                      ERROR_WRONG_DISK
#define _OSENOFCB                       ERROR_FCB_UNAVAILABLE
#define _OSESHOFLO                      ERROR_SHARING_BUFFER_EXCEEDED


/*** Origin for _OsSeekFile()                                               ***/
#define _OS_SEEKSET                     FILE_BEGIN
#define _OS_SEEKCUR                     FILE_CURRENT
#define _OS_SEEKEND                     FILE_END


/*** File attributes for _OsCreateFile()                                    ***/
#define _OS_FANORMAL                    FILE_NORMAL
#define _OS_FAREADONLY                  FILE_READONLY
#define _OS_FAHIDDEN                    FILE_HIDDEN
#define _OS_FASYSTEM                    FILE_SYSTEM


/*** File attributes for _OsOpenFile()                                      ***/
#define _OS_OAREADONLY                  OPEN_ACCESS_READONLY
#define _OS_OAWRITEONLY                 OPEN_ACCESS_WRITEONLY
#define _OS_OAREADWRITE                 OPEN_ACCESS_READWRITE


/*** Permission flags for _OsOpenFile()                                     ***/
#define _OS_SHCOMPAT                    0x00
#define _OS_SHDENYRW                    OPEN_SHARE_DENYREADWRITE
#define _OS_SHDENYWR                    OPEN_SHARE_DENYWRITE
#define _OS_SHDENYRD                    OPEN_SHARE_DENYREAD
#define _OS_SHDENYNO                    OPEN_SHARE_DENYNONE

#define _OS_NOINHERIT                   OPEN_FLAGS_NOINHERIT


#define _OsCloseFile(h)                 DosClose(h)
#define _OsCommitFile(h)                DosBufReset(h)
#define _OsCreateFile(path,h,attr)      DosOpen((PSZ)path,(PHFILE)h,NULL,0,0,FILE_CREATE,(USHORT)attr,0)
#define _OsDeleteFile(path)             DosDelete((PSZ)path,0)
#define _OsGetFilePos(h,ppos)           DosChgFilePtr(h,0,FILE_CURRENT,(PULONG)ppos)
#define _OsOpenFile(path,h,attr)        DosOpen((PSZ)path,(PHFILE)h,NULL,0,0,FILE_OPEN,(USHORT)attr,0)
#define _OsReadFile(h,len,buf,plen)     DosRead(h,(PVOID)buf,(USHORT)len,(PUSHORT)plen)
#define _OsSeekFile(h,ofs,org)          DosChgFilePtr(h,(LONG)ofs,(USHORT)org,NULL)
#define _OsWriteFile(h,len,buf,plen)    DosWrite(h,(PVOID)buf,(USHORT)len,(PUSHORT)plen)

int _OsGetFileLen(_OSHFILE h, unsigned long *plen);
int _OsLockFile(_OSHFILE h, unsigned long p, unsigned long l);
int _OsUnlockFile(_OSHFILE h, unsigned long p, unsigned long l);


/*** 32-Bit Windows only calling the Windows API                            ***/
#elif defined(_WIN) && defined(__32BIT__)
#include "windows.h"

/*** File handle type                                                       ***/
#define _OSHFILE  HANDLE


/*** Error Codes                                                            ***/
#define _OSEBADFNCODE                   ERROR_INVALID_FUNCTION
#define _OSENOENT                       ERROR_FILE_NOT_FOUND
#define _OSENOPATH                      ERROR_PATH_NOT_FOUND
#define _OSENOHND                       ERROR_TOO_MANY_OPEN_FILES
#define _OSEACCESS                      ERROR_ACCESS_DENIED
#define _OSEBADHND                      ERROR_INVALID_HANDLE
#define _OSECORMEM                      ERROR_ARENA_TRASHED
#define _OSENOMEM                       ERROR_NOT_ENOUGH_MEMORY
#define _OSEBADBLK                      ERROR_INVALID_BLOCK
#define _OSEBADENV                      ERROR_BAD_ENVIRONMENT
#define _OSEBADFMT                      ERROR_BAD_FORMAT
#define _OSEBADACC                      ERROR_INVALID_ACCESS
#define _OSEBADDAT                      ERROR_INVALID_DATA
#define _OSEBADDRV                      ERROR_INVALID_DRIVE
#define _OSECURDIR                      ERROR_CURRENT_DIRECTORY
#define _OSEBADDEV                      ERROR_NOT_SAME_DEVICE
#define _OSENOFILE                      ERROR_NO_MORE_FILES
#define _OSEWPROT                       ERROR_WRITE_PROTECT
#define _OSEBADDSK                      ERROR_I24_DEVICE_IN_USE
#define _OSENOTREADY                    ERROR_NOT_READY
#define _OSEBADCMD                      ERROR_BAD_COMMAND
#define _OSEBADCRC                      ERROR_CRC
#define _OSEBADLEN                      ERROR_BAD_LENGTH
#define _OSESEEK                        ERROR_SEEK
#define _OSBADMEDIA                     ERROR_NOT_DOS_DISK
#define _OSEBADSECT                     ERROR_SECTOR_NOT_FOUND
#define _OSENOPAPER                     ERROR_OUT_OF_PAPER
#define _OSEWRFAULT                     ERROR_WRITE_FAULT
#define _OSERDFAULT                     ERROR_READ_FAULT
#define _OSEGENFAIL                     ERROR_GEN_FAILURE
#define _OSESHRVIOL                     ERROR_SHARING_VIOLATION
#define _OSELCKVIOL                     ERROR_LOCK_VIOLATION
#define _OSEBADCHG                      ERROR_WRONG_DISK
#define _OSENOFCB                       ERROR_FCB_UNAVAILABLE
#define _OSESHOFLO                      ERROR_SHARING_BUFFER_EXCEEDED


/*** Origin for _OsSeekFile()                                               ***/
#define _OS_SEEKSET                     FILE_BEGIN
#define _OS_SEEKCUR                     FILE_CURRENT
#define _OS_SEEKEND                     FILE_END


/*** File attributes for _OsCreateFile()                                    ***/
#define _OS_FANORMAL                    0x00
#define _OS_FAREADONLY                  0x01
#define _OS_FAHIDDEN                    0x02
#define _OS_FASYSTEM                    0x03


/*** File attributes for _OsOpenFile()                                      ***/
#define _OS_OAREADONLY                  0x00
#define _OS_OAWRITEONLY                 0x01
#define _OS_OAREADWRITE                 0x02


/*** Permission flags for _OsOpenFile()                                     ***/
#define _OS_SHCOMPAT                    0x00
#define _OS_SHDENYRW                    0x10
#define _OS_SHDENYWR                    0x20
#define _OS_SHDENYRD                    0x30
#define _OS_SHDENYNO                    0x40

#define _OS_NOINHERIT                   0x80


int _OsCloseFile(_OSHFILE h);
int _OsCommitFile(_OSHFILE h);
int _OsCreateFile(const char *path, _OSHFILE *h, unsigned attr);
int _OsDeleteFile(const char *path);
int _OsGetFileLen(_OSHFILE h, unsigned long *plen);
int _OsGetFilePos(_OSHFILE h, unsigned long *ppos);
int _OsLockFile(_OSHFILE h, unsigned long p, unsigned long l);
int _OsOpenFile(const char *path, _OSHFILE *h, unsigned attr);
int _OsReadFile(_OSHFILE h, unsigned len, void *buf, unsigned *plen);
int _OsSeekFile(_OSHFILE h, unsigned ofs, unsigned org);
int _OsUnlockFile(_OSHFILE h, unsigned long p, unsigned long l);
int _OsWriteFile(_OSHFILE h, unsigned len, void *buf, unsigned *plen);
int _OsFileDate(_OSHFILE h, unsigned long *pdatetime);
int _OsSetFileDate(_OSHFILE h, unsigned long datetime);
int _OsFileTruncate(_OSHFILE h);
int _OsMakeDirectory(const char *Path);
int _OsRemoveDirectory(const char *Path);
int _OsChangeDirectory(const char *Path);
int _OsGetCurrentDir(int Drv, char *pDir/*[256]*/);
int _OsRenameFile(const char *OldPath, const char *NewPath);
int _OsFileExists(const char * FileNameAnsi);
int _OsGetFileMode(const char *Path, int *pMode);

#else
 #error "Unrecognised Target Envronment"
#endif

#if defined(__cplusplus)
}
#endif

#endif /* _OSIO_H */
